/**
 * A program for Bayesian modal estimation of an examinee ability
 * under the two-parameter logistic item characteristic curve
 * model using LSAT-6 data.
 */
import javax.swing.*; // for System.out.println
public class AppendixE1 {
  static final int NITEM = 5;
  static final double BIGT = 0.5;
  static final double HPM = 0.0; // set hyperparameters
  static final double HPVAR = 1.0;
  public static void main(String[] args) {
    double[] jirp = new double[NITEM];
      jirp[0] = 1.0; // read canned data
      jirp[1] = 0.0;
      jirp[2] = 1.0;
      jirp[3] = 0.0;
      jirp[4] = 1.0;
    double[] b = { -3.283, -1.317, -0.285, -1.766, -2.858 };
    double[] a = { 0.849, 0.758, 0.870, 0.736, 0.730 };
    int trall = -1; // set traces
    int trbme = -1;
    String yn = JOptionPane.showInputDialog
      ("Trace all? Y/N");
    if (yn.equalsIgnoreCase("y")) {
      trall = 0;
      trbme = 0;
    }
    else{
      String ynbme = JOptionPane.showInputDialog
        ("Trace BME? Y/N");
      if (ynbme.equalsIgnoreCase("y")) {
        trbme = 0;
      }
    } // end if (yn ... else
    String smaxit = JOptionPane.showInputDialog
      ("Enter maximum number of iterations to do. ");
    int maxit = Integer.parseInt(smaxit);
    System.out.println("All= " + trall +
      "  BME= " + trbme +
      "  Max Iterations= " + maxit);
    double theta = 0.0; // initial estimate of examinee's ability
    // estimate ability
    for (int nit = 1; nit <= maxit; nit++) {
      double sumnum = 0.0; // clear sums
      double sumdem = 0.0;
      System.out.println("Iteration= " + nit);
      for (int i = 0; i <= NITEM - 1; i++) { // item loop
        double uij = jirp[i];
        double dev = a[i] * (theta - b[i]);
        double phat = 1.0 / (1.0 + Math.exp(-dev));
        double wij = phat * (1.0 - phat);
        double vij = uij - phat;
        sumnum += a[i] * vij;
        sumdem += a[i] * a[i] * wij;
        if (trall == 0) {
          System.out.println("Item= " + (i + 1));
          System.out.println("Difficulty= " + b[i] +
            "  Discrimination= " + a[i] +
            "  Theta= " + theta);
          System.out.println("Uij= " + uij +
            "  Dev= " + dev +
            "  P Hat= " + phat);
          System.out.println("Wij= " + wij +
            "  Vij= " + vij);
          System.out.println("Numerator Sum= " + sumnum +
            "  Denominator Sum= " + sumdem);
        }
      } // end for (int i ... (BASIC 2190)
      sumnum -= (theta - HPM) / HPVAR;
      sumdem = - sumdem - (1.0 / HPVAR); // match equation 7.39
      if (trall == 0) {
        System.out.println("After adding proir terms");
        System.out.println("Numerator Sum= " + sumnum +
          "  Denominator Sum= " + sumdem);
      }
      double delta = sumnum / sumdem;
      if (trall == 0 || trbme == 0) {
        System.out.println("Change in Theta= " + delta);
      }
      if (Math.abs(delta) < BIGT) {
        // normal processing
      }
      else { // protect against big change in theta
        if (delta > 0.0) {
          delta = BIGT;
        }
        else {
          delta = - BIGT;
        } // end if (delta ...
      } // end if (Math.abs(delta) ...
      theta -= delta; // change subtracted as per equation 7.40
      if (trall == 0 || trbme == 0) {
        System.out.println("Theta= " + theta +
          "  Change= " + delta);
      } // end if (trall ...
      if (Math.abs(delta) < 0.05) {
        break; // exit for (int nit ...
      }
      if (nit == maxit) {
        System.out.println("Reached Max Iterations");
      }
    } // end for (int nit ... (BASIC 2460)
    System.out.println("Estimated Ability= " + theta);
    System.exit(0); // last line
  } // end public static ...
} // end public class ...
