/**
 * A program to implement Bayesian expected a posteriori ability
 * estimation using LSAT-6 data.
 */
import javax.swing.*; // for System.out.println
public class AppendixE2 {
  static final int NITEM = 5;
  static final int NQ = 10; // Number of Quadratures
  public static void main(String[] args) {
    double[] u = new double[NITEM];
      u[0] = 1.0; // read canned data
      u[1] = 0.0;
      u[2] = 1.0;
      u[3] = 0.0;
      u[4] = 1.0;
    double[] x = { -4.000, -3.111, -2.222, -1.333, -0.4444,
      0.4444, 1.333, 2.222, 3.111, 4.000 };
    double[] ak = { 0.000119, 0.002805, 0.03002, 0.1458, 0.3213,
      0.3213, 0.1458, 0.03002, 0.002805, 0.000119 };
    double[] b = { -3.283, -1.317, -0.285, -1.766, -2.858 };
    double[] a = { 0.849, 0.758, 0.870, 0.736, 0.730 };
    double[] lxk = new double[NQ];
    int trall = -1; // set traces
    int trls = -1;
    String yn = JOptionPane.showInputDialog
      ("Trace all? Y/N");
    if (yn.equalsIgnoreCase("y")) {
      trall = 0;
      trls = 0;
    }
    else{
      String ynls = JOptionPane.showInputDialog
        ("Trace L and Sums? Y/N");
      if (ynls.equalsIgnoreCase("y")) {
        trls = 0;
      }
    } // end if (yn ... else
    System.out.println("All= " + trall +
      "  L and Sums= " + trls);
    for (int k = 0; k <= NQ - 1; k++) {
      lxk[k] = 1.0;
    } // end for (int k ...
    // Bayesian expected a posteriori estimation of ability
    for (int k = 0; k <= NQ - 1; k++) {
      for (int i = 0; i <= NITEM - 1; i++) {
        double dev = - a[i] * (x[k] - b[i]); // evaluate 2pm icc
        double p = 1.0 / (1.0 + Math.exp(dev));
        double q = 1.0 - p;
        double pitem = p;
        if (u[i] == 0.0) {
          pitem = q;
        }
        lxk[k] *= pitem;
        if (trall == 0) {
          System.out.println("K= " + (k + 1) +
            "  I= " + (i + 1) +
            "  U= " + u[i] +
            "  P= " + pitem +
            "  L(X)= " + lxk[k]);
        } // end if (trall ...
      } // end for (int i ... (BASIC 370)
      if (trall == 0 || trls == 0) {
        System.out.println("K= " + (k + 1) +
          "  L(X)= " + lxk[k]);
      }
    } // end for (int k ... (BASIC 390)
    double snum = 0.0;
    double sden = 0.0;
    for (int k = 0; k <= NQ - 1; k++) {
      double la = lxk[k]*ak[k];
      double sla = x[k] * la;
      snum += xla;
      sden += la;
      if (trall == 0) {
        System.out.println("K= " + (k + 1) +
          "  LXK(K)= " + lxk[k] +
          "  AK[K]= " + ak[k] +
          "  X(K)= " + x[k]);
        System.out.println("LA= " + la +
          "  XLA= " + xla);
      } // end if (trall ...
      if (trall == 0 || trls == 0) {
        System.out.println("K= " + (k + 1) +
          "  Sum Numerator= " + snum +
          "  Sum Denominator = " + sden);
      } // end if (trall ...
    } // end for (int k ... (BASIC 510)
    double theta = snum / sden;
    System.out.println("Estimated Theta= " + theta);
    System.exit(0); // last line
  } // end public static ...
} // end public class ...
